#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// 
    /// </summary>
    [Description("Sylvain Vervoort Stocata.org Average True Range Trailing Stop")]
    public class SVEATRTrailingStop : Indicator
    {
        #region Variables
		private int 	period	= 5;
		private double	multi	= 3.5;
		
		Double	trail, loss;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Blue, PlotStyle.Line, "Trailing Stop"));
            CalculateOnBarClose	= true;
            Overlay				= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < 1)
				return;

			// Trailing stop
			loss = ATR(Input, Period)[0] * multi;
			
			if (Close[0] > Value[1] && Close[1] > Value[1])
				trail = Math.Max(Value[1], Close[0] - loss);
			
			else if (Close[0] < Value[1] && Close[1] < Value[1])
				trail = Math.Min(Value[1], Close[0] + loss);
				
			else if (Close[0] > Value[1]) trail = Close[0] - loss;
			
			else trail = Close[0] + loss;

            Value.Set(trail);
        }
		
        #region Properties
        [Description("ATR period")]
        [Category("Parameters")]
		[Gui.Design.DisplayName("1. ATR Period")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
		
		[Description("ATR multiplication Factor")]
        [Category("Parameters")]
		[Gui.Design.DisplayName("2. ATR Multiplication Factor")]
        public double Multi
        {
            get { return multi; }
            set { multi = Math.Max(0, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SVEATRTrailingStop[] cacheSVEATRTrailingStop = null;

        private static SVEATRTrailingStop checkSVEATRTrailingStop = new SVEATRTrailingStop();

        /// <summary>
        /// Sylvain Vervoort Stocata.org Average True Range Trailing Stop
        /// </summary>
        /// <returns></returns>
        public SVEATRTrailingStop SVEATRTrailingStop(double multi, int period)
        {
            return SVEATRTrailingStop(Input, multi, period);
        }

        /// <summary>
        /// Sylvain Vervoort Stocata.org Average True Range Trailing Stop
        /// </summary>
        /// <returns></returns>
        public SVEATRTrailingStop SVEATRTrailingStop(Data.IDataSeries input, double multi, int period)
        {
            if (cacheSVEATRTrailingStop != null)
                for (int idx = 0; idx < cacheSVEATRTrailingStop.Length; idx++)
                    if (Math.Abs(cacheSVEATRTrailingStop[idx].Multi - multi) <= double.Epsilon && cacheSVEATRTrailingStop[idx].Period == period && cacheSVEATRTrailingStop[idx].EqualsInput(input))
                        return cacheSVEATRTrailingStop[idx];

            lock (checkSVEATRTrailingStop)
            {
                checkSVEATRTrailingStop.Multi = multi;
                multi = checkSVEATRTrailingStop.Multi;
                checkSVEATRTrailingStop.Period = period;
                period = checkSVEATRTrailingStop.Period;

                if (cacheSVEATRTrailingStop != null)
                    for (int idx = 0; idx < cacheSVEATRTrailingStop.Length; idx++)
                        if (Math.Abs(cacheSVEATRTrailingStop[idx].Multi - multi) <= double.Epsilon && cacheSVEATRTrailingStop[idx].Period == period && cacheSVEATRTrailingStop[idx].EqualsInput(input))
                            return cacheSVEATRTrailingStop[idx];

                SVEATRTrailingStop indicator = new SVEATRTrailingStop();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Multi = multi;
                indicator.Period = period;
                Indicators.Add(indicator);
                indicator.SetUp();

                SVEATRTrailingStop[] tmp = new SVEATRTrailingStop[cacheSVEATRTrailingStop == null ? 1 : cacheSVEATRTrailingStop.Length + 1];
                if (cacheSVEATRTrailingStop != null)
                    cacheSVEATRTrailingStop.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSVEATRTrailingStop = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Sylvain Vervoort Stocata.org Average True Range Trailing Stop
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEATRTrailingStop SVEATRTrailingStop(double multi, int period)
        {
            return _indicator.SVEATRTrailingStop(Input, multi, period);
        }

        /// <summary>
        /// Sylvain Vervoort Stocata.org Average True Range Trailing Stop
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEATRTrailingStop SVEATRTrailingStop(Data.IDataSeries input, double multi, int period)
        {
            return _indicator.SVEATRTrailingStop(input, multi, period);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Sylvain Vervoort Stocata.org Average True Range Trailing Stop
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEATRTrailingStop SVEATRTrailingStop(double multi, int period)
        {
            return _indicator.SVEATRTrailingStop(Input, multi, period);
        }

        /// <summary>
        /// Sylvain Vervoort Stocata.org Average True Range Trailing Stop
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEATRTrailingStop SVEATRTrailingStop(Data.IDataSeries input, double multi, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SVEATRTrailingStop(input, multi, period);
        }
    }
}
#endregion
